@@ -134,7 +134,7 @@ setMethod("maximum", signature(x1 = "ph", x2 = "ph"),
 #' Moment Method for phase type distributions
 #'
 #' @param x an object of class \linkS4class{ph}.
-#' @param k a positive real number (moment order).
+#' @param k a positive integer (moment order).
 #' 
 #' @return The raw moment of the \linkS4class{ph} (or undelying \linkS4class{ph}) object.
 #' @export
@@ -142,16 +142,16 @@ setMethod("maximum", signature(x1 = "ph", x2 = "ph"),
 #' @examples
 #' set.seed(123)
 #' ph1 <- ph(structure = "general", dimension = 3)
-#' moment(ph1, 1.5)
+#' moment(ph1, 2)
 setMethod("moment", signature(x = "ph"), 
           function (x, k = 1){
             if(k <= 0) return("k should be positive")
+            if((k%%1) != 0) return("k should be an integer")
             if(methods::is(x, "iph")) warning("moment of undelying ph structure is provided for iph objects")
             m <- solve(-x@pars$S)
-            p <- eigen(m)$vectors
-            d <- diag(eigen(m)$values^{k})
-            return(sum(gamma(k + 1) * x@pars$alpha %*% p %*% d %*% solve(p)))
-            
+            prod <- diag(nrow(m))
+            for(i in 1:k){prod <- prod %*% m}
+            return(factorial(k)*sum(x@pars$alpha %*% prod))
           }
 )
 
